# Generated by Django 6.0 on 2025-12-29 06:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('saas_base', '0003_remove_member_is_staff'),
    ]

    operations = [
        migrations.AddField(
            model_name='tenant',
            name='logo',
            field=models.URLField(blank=True, null=True),
        ),
    ]
