.button {
  box-sizing: border-box;
  width: 100%;
  padding-top: 15px;
  padding-bottom: 15px;
}

.button table {
  width: auto;
}

.button table td {
  background-color: #1e1e1e;
  border-radius: 5px;
  text-align: center;
}

.button a {
  background-color: #1e1e1e;
  color: #ffffff;
  border: solid 1px #1e1e1e;
  border-radius: 5px;
  box-sizing: border-box;
  cursor: pointer;
  display: inline-block;
  font-size: 14px;
  font-weight: bold;
  margin: 0;
  padding: 12px 25px;
  text-decoration: none;
  text-transform: capitalize;
}

.auth-code {
  box-sizing: border-box;
  width: 100%;
  padding-top: 16px;
  padding-bottom: 16px;
}

.auth-code > tbody > tr > td {
  padding-bottom: 15px;
}

.auth-code table {
  width: auto;
}

.auth-code table td {
  border-radius: 5px;
  text-align: center;
}

.auth-code table td > code {
  padding: 12px 25px;
  border: dashed 2px #ddd;
  background-color: #f0f0f0;
  border-radius: 5px;
  font-size: 18px;
  letter-spacing: 1px;
}
