img {
  border: none;
  -ms-interpolation-mode: bicubic;
  max-width: 100%;
}

body {
  background-color: #F9F9F9;
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Oxygen, Ubuntu, Droid Sans, Helvetica Neue, sans-serif;
  -webkit-font-smoothing: antialiased;
  font-size: 16px;
  line-height: 1.4;
  margin: 0;
  padding: 0;
  -ms-text-size-adjust: 100%;
  -webkit-text-size-adjust: 100%;
}

table {
  border-collapse: separate;
  mso-table-lspace: 0;
  mso-table-rspace: 0;
  width: 100%;
}

table td {
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Oxygen, Ubuntu, Droid Sans, Helvetica Neue, sans-serif;
  font-size: 16px;
  vertical-align: top;
}

/* -------------------------------------
    TYPOGRAPHY
------------------------------------- */
h1,
h2,
h3,
h4 {
  color: #000000;
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Oxygen, Ubuntu, Droid Sans, Helvetica Neue, sans-serif;
  font-weight: 400;
  line-height: 1.4;
  margin: 0 0 30px 0;
}

h1 {
  font-size: 32px;
  font-weight: 300;
  text-align: center;
  text-transform: capitalize;
}

p,
ul,
ol {
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Oxygen, Ubuntu, Droid Sans, Helvetica Neue, sans-serif;
  font-size: 16px;
  font-weight: normal;
  margin: 0 0 15px 0;
}

ul,
ol {
  padding-left: 10px;
}

p li,
ul li,
ol li {
  list-style-position: inside;
  margin-left: 5px;
}

a {
  color: #0588F0;
  text-decoration: underline;
}

hr {
  border: 0;
  border-bottom: 1px solid #F9F9F9;
  margin: 20px 0;
}
