<!doctype html>
<html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  <title>{% block title %}{% endblock %}</title>
  <style>
    {% include "saas_emails/partials/reset.css" %}
    {% include "saas_emails/partials/layout.css" %}
    {% include "saas_emails/partials/components.css" %}
  </style>
  <style data-css-inline="keep">
    {% include "saas_emails/partials/responsive.css" %}
  </style>
</head>
<body>
<table role="presentation" border="0" cellpadding="0" cellspacing="0" class="body">
  <tr>
    <td>&nbsp;</td>
    <td class="container">
      <div class="content">
        <table role="presentation" class="main">
          <tr>
            <td class="wrapper">
              <table role="presentation" border="0" cellpadding="0" cellspacing="0">
                <tr>
                  <td>
                    {% block content %}{% endblock %}
                  </td>
                </tr>
              </table>
            </td>
          </tr>
        </table>
        <div class="footer">
          <table role="presentation" border="0" cellpadding="0" cellspacing="0">
            {% block footer %}
              {% include "saas_emails/partials/footer.html" %}
            {% endblock %}
          </table>
        </div>
      </div>
    </td>
    <td>&nbsp;</td>
  </tr>
</table>
</body>
</html>
