{% extends "saas_emails/base.html" %}

{% block title %}Reset Password{% endblock %}

{% block content %}
  <p>Hello {{ user }},</p>

  <p>We have received a request to reset your password. To ensure the security of your account, please use the code provided below:</p>

  <p>Your Reset Code:</p>

  {% include "saas_emails/partials/auth_code.html" %}

  <p>If you didn't initiate this request, kindly disregard this email. Rest assured that your account remains safe and secure.</p>
{% endblock %}
