# Generated by Django 5.2.6 on 2025-11-20 08:00

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('saas_base', '0002_add_role_and_update_member'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='member',
            name='is_staff',
        ),
    ]
